@@ -118,6 +118,14 @@ ncclResult_t ncclAllGather(...) {
     return ncclSuccess;
 }
 
+ncclResult_t ncclSend(...) {
+    return ncclSuccess;
+}
+
+ncclResult_t ncclRecv(...) {
+    return ncclSuccess;
+}
+
 typedef struct CUstream_st *cudaStream_t;
 
 }  // extern "C"
@@ -168,6 +176,15 @@ ncclDataType_t _get_proper_datatype(ncclDataType_t datatype) {
     return TYPE2TYPE_V1[datatype];
 }
 
+#ifndef CUPY_NO_CUDA
+ncclResult_t ncclGroupStart() {
+    return ncclSuccess;
+}
+
+ncclResult_t ncclGroupEnd() {
+    return ncclSuccess;
+}
+#endif // #ifndef CUPY_NO_CUDA
 #endif // #if (NCCL_VERSION_CODE < 2000)
 
 #if (NCCL_VERSION_CODE < 2200)
@@ -188,18 +205,6 @@ ncclResult_t ncclGetVersion(int *version) {
 
 #endif // #if (NCCL_VERSION_CODE < 2304)
 
-#ifndef CUPY_NO_CUDA
-#if (NCCL_VERSION_CODE < 2000)
-ncclResult_t ncclGroupStart() {
-    return ncclSuccess;
-}
-
-ncclResult_t ncclGroupEnd() {
-    return ncclSuccess;
-}
-#endif // #if (NCCL_VERSION_CODE < 2200)
-#endif // #ifndef CUPY_NO_CUDA
-
 ncclResult_t _ncclAllReduce(const void* sendbuff, void* recvbuff, size_t count,
                             ncclDataType_t datatype, ncclRedOp_t op, ncclComm_t comm,
                             cudaStream_t stream) {
@@ -263,4 +268,17 @@ void ncclCommAbort(ncclComm_t comm) {
 }
 #endif
 
+#if (NCCL_VERSION_CODE < 2700)
+// New functions in 2.7
+ncclResult_t ncclSend(const void* sendbuff, size_t count, ncclDataType_t datatype,
+                      int peer, ncclComm_t comm, cudaStream_t stream) {
+    return ncclSuccess;
+}
+
+ncclResult_t ncclRecv(void* recvbuff, size_t count, ncclDataType_t datatype,
+                      int peer, ncclComm_t comm, cudaStream_t stream) {
+    return ncclSuccess;
+}
+#endif
+
 #endif // #ifndef INCLUDE_GUARD_CUPY_NCCL_H
