@@ -10,19 +10,21 @@ import tempfile
 from install import utils
 
 
-PLATFORM_DARWIN = sys.platform.startswith('darwin')
 PLATFORM_LINUX = sys.platform.startswith('linux')
 PLATFORM_WIN32 = sys.platform.startswith('win32')
 
-minimum_cuda_version = 8000
-minimum_cudnn_version = 5000
-maximum_cudnn_version = 7999
+minimum_cuda_version = 9000
+minimum_cudnn_version = 7000
+
+minimum_hip_version = 305  # for ROCm 3.5.0+
 
 _cuda_path = 'NOT_INITIALIZED'
 _rocm_path = 'NOT_INITIALIZED'
 _compiler_base_options = None
 
 
+# Using tempfile.TemporaryDirectory would cause an error during cleanup
+# due to a bug: https://bugs.python.org/issue26660
 @contextlib.contextmanager
 def _tempdir():
     temp_dir = tempfile.mkdtemp()
@@ -144,6 +146,7 @@ def get_compiler_setting(use_hip):
 
     if rocm_path:
         include_dirs.append(os.path.join(rocm_path, 'include'))
+        include_dirs.append(os.path.join(rocm_path, 'include', 'hip'))
         include_dirs.append(os.path.join(rocm_path, 'rocrand', 'include'))
         library_dirs.append(os.path.join(rocm_path, 'lib'))
         library_dirs.append(os.path.join(rocm_path, 'rocrand', 'lib'))
@@ -151,9 +154,6 @@ def get_compiler_setting(use_hip):
     if use_hip:
         extra_compile_args.append('-std=c++11')
 
-    if PLATFORM_DARWIN:
-        library_dirs.append('/usr/local/cuda/lib')
-
     if PLATFORM_WIN32:
         nvtoolsext_path = os.environ.get('NVTOOLSEXT_PATH', '')
         if os.path.exists(nvtoolsext_path):
@@ -162,13 +162,34 @@ def get_compiler_setting(use_hip):
         else:
             define_macros.append(('CUPY_NO_NVTX', '1'))
 
-    cub_path = os.environ.get('CUB_PATH', '')
-    if os.path.exists(cub_path):
-        # for <cupy/complex.cuh>
-        cupy_header = os.path.join(os.path.dirname(os.path.realpath(__file__)),
-                                   '../cupy/core/include')
-        include_dirs.append(cupy_header)
-        include_dirs.append(cub_path)
+    # For CUB, we need the complex and CUB headers. The search precedence for
+    # the latter is:
+    #   1. built-in CUB (for CUDA 11+ and ROCm)
+    #   2. CuPy's CUB bundle
+    # Note that starting CuPy v8 we no longer use CUB_PATH
+
+    # for <cupy/complex.cuh>
+    cupy_header = os.path.join(os.path.dirname(os.path.realpath(__file__)),
+                               '../cupy/core/include')
+    if cuda_path:
+        cuda_cub_path = os.path.join(cuda_path, 'include', 'cub')
+        if not os.path.exists(cuda_cub_path):
+            cuda_cub_path = None
+    elif rocm_path:
+        cuda_cub_path = os.path.join(rocm_path, 'include', 'hipcub')
+        if not os.path.exists(cuda_cub_path):
+            cuda_cub_path = None
+    else:
+        cuda_cub_path = None
+    global _cub_path
+    if cuda_cub_path:
+        _cub_path = cuda_cub_path
+    elif not use_hip:  # CuPy's bundle doesn't work for ROCm
+        _cub_path = os.path.join(cupy_header, 'cupy', 'cub')
+    else:
+        raise Exception('Please install hipCUB and retry')
+    include_dirs.insert(0, _cub_path)
+    include_dirs.insert(1, cupy_header)
 
     return {
         'include_dirs': include_dirs,
@@ -201,28 +222,28 @@ def _match_output_lines(output_lines, regexs):
     return None
 
 
-def get_compiler_base_options():
+def get_compiler_base_options(compiler_path):
     """Returns base options for nvcc compiler.
 
     """
     global _compiler_base_options
     if _compiler_base_options is None:
-        _compiler_base_options = _get_compiler_base_options()
+        _compiler_base_options = _get_compiler_base_options(compiler_path)
     return _compiler_base_options
 
 
-def _get_compiler_base_options():
+def _get_compiler_base_options(compiler_path):
     # Try compiling a dummy code.
     # If the compilation fails, try to parse the output of compilation
     # and try to compose base options according to it.
-    nvcc_path = get_nvcc_path()
+    # compiler_path is the path to nvcc (CUDA) or hipcc (ROCm/HIP)
     with _tempdir() as temp_dir:
         test_cu_path = os.path.join(temp_dir, 'test.cu')
         test_out_path = os.path.join(temp_dir, 'test.out')
         with open(test_cu_path, 'w') as f:
             f.write('int main() { return 0; }')
         proc = subprocess.Popen(
-            nvcc_path + ['-o', test_out_path, test_cu_path],
+            compiler_path + ['-o', test_out_path, test_cu_path],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE)
         stdoutdata, stderrdata = proc.communicate()
@@ -251,8 +272,13 @@ def _get_compiler_base_options():
 
 
 _cuda_version = None
+_hip_version = None
+_thrust_version = None
 _cudnn_version = None
 _nccl_version = None
+_cutensor_version = None
+_cub_path = None
+_cub_version = None
 
 
 def check_cuda_version(compiler, settings):
@@ -276,16 +302,12 @@ def check_cuda_version(compiler, settings):
     if _cuda_version < minimum_cuda_version:
         utils.print_warning(
             'CUDA version is too old: %d' % _cuda_version,
-            'CUDA v7.0 or newer is required')
+            'CUDA 9.0 or newer is required')
         return False
 
     return True
 
 
-def _format_cuda_version(version):
-    return str(version)
-
-
 def get_cuda_version(formatted=False):
     """Return CUDA Toolkit version cached in check_cuda_version()."""
     global _cuda_version
@@ -293,10 +315,81 @@ def get_cuda_version(formatted=False):
         msg = 'check_cuda_version() must be called first.'
         raise RuntimeError(msg)
     if formatted:
-        return _format_cuda_version(_cuda_version)
+        return str(_cuda_version)
     return _cuda_version
 
 
+def check_hip_version(compiler, settings):
+    global _hip_version
+    try:
+        out = build_and_run(compiler, '''
+        #include <hip/hip_version.h>
+        #include <stdio.h>
+        int main() {
+          printf("%d", HIP_VERSION);
+          return 0;
+        }
+        ''', include_dirs=settings['include_dirs'])
+
+    except Exception as e:
+        utils.print_warning('Cannot check HIP version', str(e))
+        return False
+
+    _hip_version = int(out)
+
+    if _hip_version < minimum_hip_version:
+        utils.print_warning(
+            'ROCm/HIP version is too old: %d' % _hip_version,
+            'ROCm 3.5.0 or newer is required')
+        return False
+
+    return True
+
+
+def get_hip_version(formatted=False):
+    """Return ROCm version cached in check_hip_version()."""
+    global _hip_version
+    if _hip_version is None:
+        msg = 'check_hip_version() must be called first.'
+        raise RuntimeError(msg)
+    if formatted:
+        return str(_hip_version)
+    return _hip_version
+
+
+def check_thrust_version(compiler, settings):
+    global _thrust_version
+
+    try:
+        out = build_and_run(compiler, '''
+        #include <thrust/version.h>
+        #include <stdio.h>
+
+        int main() {
+          printf("%d", THRUST_VERSION);
+          return 0;
+        }
+        ''', include_dirs=settings['include_dirs'])
+    except Exception as e:
+        utils.print_warning('Cannot check Thrust version\n{0}'.format(e))
+        return False
+
+    _thrust_version = int(out)
+
+    return True
+
+
+def get_thrust_version(formatted=False):
+    """Return Thrust version cached in check_thrust_version()."""
+    global _thrust_version
+    if _thrust_version is None:
+        msg = 'check_thrust_version() must be called first.'
+        raise RuntimeError(msg)
+    if formatted:
+        return str(_thrust_version)
+    return _thrust_version
+
+
 def check_cudnn_version(compiler, settings):
     global _cudnn_version
     try:
@@ -315,13 +408,11 @@ def check_cudnn_version(compiler, settings):
 
     _cudnn_version = int(out)
 
-    if not minimum_cudnn_version <= _cudnn_version <= maximum_cudnn_version:
-        min_major = _format_cuda_version(minimum_cudnn_version)
-        max_major = _format_cuda_version(maximum_cudnn_version)
+    if not minimum_cudnn_version <= _cudnn_version:
+        min_major = str(minimum_cudnn_version)
         utils.print_warning(
-            'Unsupported cuDNN version: {}'.format(
-                _format_cuda_version(_cudnn_version)),
-            'cuDNN v{}= and <=v{} is required'.format(min_major, max_major))
+            'Unsupported cuDNN version: {}'.format(str(_cudnn_version)),
+            'cuDNN >=v{} is required'.format(min_major))
         return False
 
     return True
@@ -334,7 +425,7 @@ def get_cudnn_version(formatted=False):
         msg = 'check_cudnn_version() must be called first.'
         raise RuntimeError(msg)
     if formatted:
-        return _format_cuda_version(_cudnn_version)
+        return str(_cudnn_version)
     return _cudnn_version
 
 
@@ -378,7 +469,7 @@ def get_nccl_version(formatted=False):
     if formatted:
         if _nccl_version == 0:
             return '1.x'
-        return _format_cuda_version(_nccl_version)
+        return str(_nccl_version)
     return _nccl_version
 
 
@@ -401,6 +492,84 @@ def check_nvtx(compiler, settings):
     return True
 
 
+def check_cub_version(compiler, settings):
+    global _cub_version
+    global _cub_path
+
+    # This is guaranteed to work for any CUB source because the search
+    # precedence follows that of include paths.
+    # - On CUDA, CUB < 1.9.9 does not provide version.cuh and would error out
+    # - On ROCm, hipCUB has the same version as rocPRIM (as of ROCm 3.5.0)
+    try:
+        out = build_and_run(compiler,
+                            '''
+                            #ifndef CUPY_USE_HIP
+                            #include <cub/version.cuh>
+                            #else
+                            #include <hipcub/hipcub_version.hpp>
+                            #endif
+                            #include <stdio.h>
+
+                            int main() {
+                              #ifndef CUPY_USE_HIP
+                              printf("%d", CUB_VERSION);
+                              #else
+                              printf("%d", HIPCUB_VERSION);
+                              #endif
+                              return 0;
+                            }''',
+                            include_dirs=settings['include_dirs'],
+                            define_macros=settings['define_macros'])
+    except Exception as e:
+        # could be in a git submodule?
+        try:
+            # CuPy's bundle
+            cupy_cub_include = _cub_path
+            a = subprocess.run(' '.join(['git', 'describe', '--tags']),
+                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                               shell=True, cwd=cupy_cub_include)
+            if a.returncode == 0:
+                tag = a.stdout.decode()[:-1]
+
+                # CUB's tag convention changed after 1.8.0: "v1.9.0" -> "1.9.0"
+                # In any case, we normalize it to be in line with CUB_VERSION
+                if tag.startswith('v'):
+                    tag = tag[1:]
+                tag = tag.split('.')
+                out = int(tag[0]) * 100000 + int(tag[1]) * 100
+                try:
+                    out += int(tag[2])
+                except ValueError:
+                    # there're local commits so tag is like 1.8.0-1-gdcbb288f,
+                    # we add the number of commits to the version
+                    local_patch = tag[2].split('-')
+                    out += int(local_patch[0]) + int(local_patch[1])
+            else:
+                raise RuntimeError('Cannot determine CUB version from git tag'
+                                   '\n{0}'.format(e))
+        except Exception as e:
+            utils.print_warning('Cannot determine CUB version\n{0}'.format(e))
+            # 0: CUB is not built (makes no sense), -1: built with unknown ver
+            out = -1
+
+    _cub_version = int(out)
+    settings['define_macros'].append(('CUPY_CUB_VERSION_CODE', _cub_version))
+    return True  # we always build CUB
+
+
+def get_cub_version(formatted=False):
+    """Return CUB version cached in check_cub_version()."""
+    global _cub_version
+    if _cub_version is None:
+        msg = 'check_cub_version() must be called first.'
+        raise RuntimeError(msg)
+    if formatted:
+        if _cub_version == -1:
+            return '<unknown>'
+        return str(_cub_version)
+    return _cub_version
+
+
 def check_cutensor_version(compiler, settings):
     global _cutensor_version
     try:
